#!/usr/bin/env perl
use 5.016;
use ZMQ::LibZMQ3;
use ZMQ::Constants qw(ZMQ_PUB ZMQ_FD ZMQ_SNDMORE);
use AnyEvent;

my $context = zmq_init();
my $publisher = zmq_socket($context, ZMQ_PUB);
zmq_bind($publisher, 'tcp://*:10001');

say "Collect MP3 filenames";
my $data = get_mp3_data();

say "Start publishing messages";
my $w = AnyEvent->timer(after => 1, interval => 0.1, cb => sub {
    my $message = $data->[rand(@$data)];
    my $last_frame = pop(@$message);
    foreach my $frame (@$message) {
        zmq_msg_send($frame, $publisher, ZMQ_SNDMORE);
    }
    zmq_msg_send($last_frame, $publisher);
});
AnyEvent->condvar->recv;

sub get_mp3_data {
    use Path::Iterator::Rule;
    my $rule = Path::Iterator::Rule->new->file->name('*.mp3');
    my @all;
    my $next = $rule->iter( '/home/domm/media/mp3', {relative=>1});
    while ( my $file = $next->() ) {
        my ($artist, $record, $track)=split(/\//,$file,3);
        next unless $track && $track =~ /\.mp3$/;
        push(@all,[
            $file,
            $artist || 'unknown artist',
            $record || 'unknown record',
            $track
        ]);
    }
    return \@all;
}
