#!/usr/bin/env perl
use 5.016;
use ZMQx::Class;
use ZMQx::Class::AnyEvent;
use ZMQ::Constants 'ZMQ_DONTWAIT';

my $control = ZMQx::Class->socket('ROUTER', bind=>'tcp://*:10001');
my $fan     = ZMQx::Class->socket('PUSH',   bind=>'tcp://*:10002');
my $sink    = ZMQx::Class->socket('SUB',    bind=>'tcp://*:10003');
my %pending;

# handle requests coming in via control socket (ROUTER)
say "setting up control socket handler";
my $control_in = ZMQx::Class::AnyEvent->watcher( $control, sub {
    while (my $message = $control->receive) {
        my ($address, $null, @tasks) = @$message;

        my $task_id = sprintf("r%09d", rand(900_000_000));
        my $task_count = @tasks;
        my @result = map { '' } @tasks;
        $pending{$task_id} = {
            expect   => $task_count,
            reply_to => $address,
            results  => \@result,
        };

        say "$task_id: Ventilator got: ".join( ',', @tasks );

        # tell workers to work
        for my $index (0 .. $task_count-1) {
            my $task = $tasks[$index];
            say "$task_id: send request to worker: $index:$task";
            $fan->send([$task_id,$index,$task]);
        }
    }
});

## handle replies coming in via the sink (SUB)
say "setting up sink socket handler";
$sink->subscribe('');
my $sink_in = ZMQx::Class::AnyEvent->watcher( $sink, sub {
    while (my $message = $sink->receive) {
        my ($task_id, $index, $word, $cost, $pid) = @$message;

        my $p = $pending{$task_id};
        $p->{results}[$index] = "$word:$cost:$pid";

        if ((scalar grep { $_ } @{$p->{results}}) == $p->{expect}) {
            say "$task_id: got all results back";
            delete $pending{$task_id};
            $control->send([ $p->{reply_to},'',@{ $p->{results} } ]);
        }
    }
});

AnyEvent->condvar->recv;
