#!/usr/bin/env perl
use 5.016;
use ZMQx::Class;
use ZMQx::Class::AnyEvent;
use ZMQ::Constants 'ZMQ_DONTWAIT';

my $control = ZMQx::Class->socket('ROUTER', bind=>'tcp://*:10001');
my $fan     = ZMQx::Class->socket('PUSH',   bind=>'tcp://*:10002');
my $sink    = ZMQx::Class->socket('SUB',    bind=>'tcp://*:10003');
my %pending;

# handle requests coming in via control socket (ROUTER)
say "setting up control socket handler";
my $control_in = ZMQx::Class::AnyEvent->watcher( $control, sub {
    while (my $message = $control->receive) {
        my ($address, $null, @tasks) = @$message;

        my $task_id = sprintf("r%09d", rand(900_000_000));
        my $task_count = @tasks;
        my @result = map { '' } @tasks;

        say "$task_id: Ventilator got: ".join( ',', @tasks );

        $pending{$task_id} = {
            expect   => $task_count,
            reply_to => $address,
            results  => \@result,
            timeout  => undef,
        };

        # setup timeout handler
        $pending{$task_id}{timeout} = AnyEvent->timer(after => 5, cb => sub {
            say "$task_id: TIMEOUT!";
            send_result($task_id);
        });

        # tell workers to work
        for my $index (0 .. $task_count-1) {
            my $task = $tasks[$index];
            say "$task_id: send request to worker: $index:$task";
            $fan->send([$task_id,$index,$task]);
        }
    }
});

## handle replies coming in via the sink (SUB)
say "setting up sink socket handler";
$sink->subscribe('');
my $sink_in = ZMQx::Class::AnyEvent->watcher( $sink, sub {
    while (my $message = $sink->receive) {
        my ($task_id, $index, $word, $cost, $pid) = @$message;

        if (exists $pending{$task_id}) {
            my $p = $pending{$task_id};
            $p->{results}[$index] = "$word:$cost:$pid";

            if ((scalar grep { $_ } @{$p->{results}}) == $p->{expect}) {
                say "$task_id: got all results back";
                send_result($task_id);
            }
        }
        else {
            say "Got orphaned message $task_id:$index";
        }

    }
});

AnyEvent->condvar->recv;

sub send_result {
    my $task_id = shift;

    my $task = delete $pending{$task_id};
    delete $task->{timeout};
    my @results = @{$task->{results}};

    say "$task_id: send reply to client";
    if (@results) {
        $control->send([ $task->{reply_to},'',@results ]);
    }
    else {
        $control->send([ $task->{reply_to},'','no-data' ]);
    }
}

